#! /usr/bin/env ruby

require 'curl'

=begin

= yahoo_finance.rb

== USAGE

This library is useful to download and manipulate Financial indexes, stocks, quote data from the Yahoo web site

Examples


  #!/usr/bin/env ruby
  require 'yahoo_finance'
  symbol = "FIA.MI" 
  p YahooSymbol.new( symbol ).getQuote


  #!/usr/bin/env ruby
  require 'yahoo_finance'
  index = ARGV[0] || "^DJI"
  YahooIndex.new( index ).getSymbols.each { |s|
	  p s.getQuote['Symbol']
  }

== INSTALLATION

Copy this file in the rubylib directory (for example  "/usr/lib/ruby/1.6")


== REQUIREMENTS

I've tested this library with a Debian Linux 3.0 and
  * ruby 1.6.7
  * rbCurl >= 0.0.2 (http://www.d1.dion.ne.jp/~matuyuki/ruby.html)

== HISTORY

  * 2002-08-14 (Matteo) - first rows of this file


== AUTHOR

E-MAIL: matteo.redaelli@libero.it

WEB:    http://digilander.iol.it/reda

== LICENSE

This package is free software; you can redistribute it and/or
modify it under the same terms as Perl itself, i.e., under the
terms of the "Artistic License" or the "GNU General Public License".

The C library at the core of this Perl module can additionally
be redistributed and/or modified under the terms of the
"GNU Library General Public License".


== DISCLAIMER

This package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the "GNU General Public License" for more detail



=end

INDEX_SYMBOLS_URL = 'http://finance.yahoo.com/d/quotes.csv?s=@__INDEX__&f=sl1d1t1c1ohgv&e=.csv'

SYMBOL_DETAIL_QUOTE_URL = 'http://quote.yahoo.com/d?f=snl1d1t1c1p2va2bapomwerr1dyj1x&s='

SYMBOL_ATTRIBUTES = [
	"Symbol",
    "Company Name",
    "Last Price",
    "Last Trade Date",
    "Last Trade Time",
    "Change",
    "Percent Change",
    "Volume",
    "Average Daily Vol",
    "Bid",
    "Ask",
    "Previous Close",
    "Today's Open",
    "Day's Range",
    "52-Week Range",
    "Earnings per Share",
    "P/E Ratio",
    "Dividend Pay Date",
    "Dividend per Share",
    "Dividend Yield",
    "Market Capitalization",
    "Stock Exchange"]

class YahooSymbolQuote < Hash 
	def initialize ( attribute = nil)
	  super()
	  attribute and SYMBOL_ATTRIBUTES.reverse_each { |attr|
		self[attr] = attribute.pop
	  }
	end
	def get_attribute( attr )
		self[ attr ]
	end
		
end

module WWWCurl
	def getWWWPage( url )
		w = Curl.new
		w.followlocation = true
		w.timeout = 20
		w.url = url
		w.perform
		w.body_str
	end
end

class YahooSymbol
	attr_writer :quote_url
	attr_reader :symbol

	include WWWCurl

	def initialize( symbol )
		@symbol = symbol
		@quote_url = SYMBOL_DETAIL_QUOTE_URL
	end
	def to_s
		"Symbol: #{@symbol}"
	end
	def getQuote
		url = @quote_url + @symbol
		body = getWWWPage( url )
		YahooSymbolQuote.new( (body.chomp.gsub(/"/,'').sub(/\r$/,'').split(',') ) )
	end
end

class YahooIndex < YahooSymbol
	attr_writer :index_symbols_url

	def initialize( symbol )
		super( symbol )
		@index_symbols_url = INDEX_SYMBOLS_URL
	end

	def getSymbols
		url = @index_symbols_url.sub(/__INDEX__/, @symbol)
		body = getWWWPage( url )
		quotes = Array.new;
		body.each_line { |line| 
		  q = line.chomp.gsub(/"/,'').sub(/\r$/,'').split(',')
		  q[0] and   quotes.push( YahooSymbol.new(q[0]) )
		}
		quotes
	end
	def getSymbolsQuotes
		getSymbols.collect{ |sym| sym.getQuote	}
	end
end

