#! /usr/bin/env ruby


=begin

= yahoo_finance_db.rb

== USAGE

This library uses the module "yahoo_finance.rb" to download indexes, stockes, symbols, quote data
from the Yahoo web site and then add them in a MySQL database. The module 'ruby-dbi' is used and so it
shouldn't be hard to use an other database (postgres, interbase, oracle, ...)

Examples:

To add a new index (new entries in the tables yahoo_index, yahoo_symbol, yahoo_index_simbol) 


require "yahoo_finance_db"
index = "^DJI"
a = YahooFinanceDB.new('DBI:Mysql:yahoo_finance', 'root','')
i = YahooIndex.new( index ) 
a.addIndex( i )


== INSTALLATION

Copy this file in the rubylib directory (for example  "/usr/lib/ruby/1.6")

Create a new MySql database with the name "yahoo_finance"

        mysqladmin -u root create yahoo_finance

and populate it with

        mysql -u root yahoo_finance <yahoo_finance.sql

== REQUIREMENTS


I've tested this library with a Debian Linux 3.0 and
  * ruby 1.6.7
  * ruby-dbi >= 0.0.16 (http://ruby-dbi.sourceforge.net)
  * mysql server  3.23.49

See yahoo_finance.rb for other requirements


== TO DO

A web interface to see the data stored in the database and to build some useful charts


== HISTORY

  * 2002-08-14 (Matteo) - first rows of this file


== AUTHOR

E-MAIL: matteo.redaelli@libero.it

WEB:    http://digilander.iol.it/reda

== LICENSE

This package is free software; you can redistribute it and/or
modify it under the same terms as Perl itself, i.e., under the
terms of the "Artistic License" or the "GNU General Public License".

The C library at the core of this Perl module can additionally
be redistributed and/or modified under the terms of the
"GNU Library General Public License".


== DISCLAIMER

This package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the "GNU General Public License" for more detail




=end

require 'dbi'
require 'yahoo_finance'

class YahooFinanceDB
	def initialize( db_driver_url='DBI:Mysql:yahoo_finance', db_user='root', db_pwd='', db_params=nil )
		@db_driver_url = db_driver_url
		@db_user = db_user
		@db_pwd = db_pwd
		@db_params = db_params
	end


	def db_connect
		DBI.connect(@db_driver_url, @db_user, @db_pwd, @db_params)
	end

	def db_disconnect( dbh )
		dbh.disconnect
	end

	def db_ddl(sql)
		begin
		  dbh = db_connect
		  dbh.do(sql)
		  db_disconnect( dbh )
		rescue
			s = sql.gsub(/\n/,'').gsub(/\t/,'')
			p "KO ** #{s} **"
		end
	end

	def db_select(sql)
		dbh = db_connect
		sth=dbh.prepare(sql)
		sth.execute
		rows = sth.fetch_all
		db_disconnect( dbh )
		rows
	end

	def addSymbol( symbol )
		data = symbol.getQuote
		sql = "insert into yahoo_symbol (
				symbol_id,
				symbol_desc,
				company_name,
				stock_exchange 
			) values ( 
				'#{data['Symbol']}', 
				'#{data['Symbol']}', 
				'#{data['Company Name']}',
				'#{data['Stock Exchange']}'
			)"
		db_ddl(sql)
	end

	def data2mysql( data)
		if data === "N/A"
			"0000-00-00"
		else
		  d  = data.split('/')
		  d[2] + '-' + d[0] + '-' + d[1]
		end
	end
	def addSymbolQuote( symbol )
		p symbol.symbol
		data = symbol.getQuote

		# the data field must be changed for mysql

		day = data2mysql( data['Last Trade Date'] )
		
		sql = "select 1 from yahoo_quote where
			symbol_id = '#{data['Symbol']}' and
			the_day = '#{day}'"
		
		if  db_select(sql) 
		  sql = "update yahoo_quote set
				  the_time = '#{data['Last Trade Time']}',
				  quote	= '#{data['Last Price']}'
			  where
				  symbol_id = '#{data['Symbol']}' and
				  the_day = '#{day}' 
			  "
		else
		  sql = "insert into yahoo_quote (
				  symbol_id,
				  the_day,
				  the_time,
				  quote	
			  ) values ( 
				  '#{data['Symbol']}', 
				  '#{day}', 
				  '#{data['Last Trade Time']}',
				  '#{data['Last Price']}'
			  )"
		end
		db_ddl(sql)
	end
	def addAllQuote
		sql = "select symbol_id from yahoo_symbol"

		rows  = db_select( sql )
		rows.each { |r|
			addSymbolQuote( YahooSymbol.new(r[0]) )

		}
	end

	def addIndex( index )

		# now I'm going to add the symbols of that index
		# in the tables yahoo_index_symbol and yahoo_symbol

		data = index.getQuote

		index.getSymbols.each { |s|
			if s.symbol === "@" + index.symbol
				p "Symbols of the index are not available -- exiting..."
				return
			end

			addSymbol( s )
			sql = "insert into yahoo_index_symbol (
				index_id,
				symbol_id
			) values ( 
				'#{data['Symbol']}', 
				'#{s.getQuote['Symbol']}' 
			)"
			db_ddl(sql)
		}
		sql = "insert into yahoo_index (
				index_id,
				index_desc
			) values ( 
				'#{data['Symbol']}', 
				'#{data['Symbol']}' 
			)"
		db_ddl(sql)
	end
end


